<?php

namespace Drupal\config_notify;

use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\ImportStorageTransformer;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Config\StorageComparer;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class NotifierService.
 */
class NotifierService {
  use StringTranslationTrait;

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Drupal\Core\Config\ImportStorageTransformer definition.
   *
   * @var \Drupal\Core\Config\ImportStorageTransformer
   */
  protected $configImportTransformer;

  /**
   * Drupal\Core\Config\StorageInterface definition.
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $configStorageSync;

  /**
   * Drupal\Core\Datetime\DateFormatterInterface definition.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Drupal\Core\Extension\ModuleHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Drupal\Core\State\StateInterface definition.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Drupal\Core\Mail\MailManagerInterface definition.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $pluginManagerMail;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Drupal\Core\Language\LanguageManagerInterface definition.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new NotifierService object.
   */
  public function __construct(MessengerInterface $messenger, ConfigManagerInterface $config_manager, ImportStorageTransformer $config_import_transformer, StorageInterface $config_storage_sync, StorageInterface $config_storage_active, DateFormatterInterface $date_formatter, ModuleHandlerInterface $module_handler, StateInterface $state, MailManagerInterface $plugin_manager_mail, ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager) {
    $this->messenger = $messenger;
    $this->configManager = $config_manager;
    $this->configImportTransformer = $config_import_transformer;
    $this->configStorageSync = $config_storage_sync;
    $this->configStorageActive = $config_storage_active;
    $this->dateFormatter = $date_formatter;
    $this->moduleHandler = $module_handler;
    $this->state = $state;
    $this->pluginManagerMail = $plugin_manager_mail;
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
  }

  /**
   * Check for changes in active configuration.
   *
   * @return bool
   *   Whether there are changes on the active configuration or not.
   */
  public function checkChanges() {
    $syncStorage = $this->configImportTransformer->transform($this->configStorageSync);
    $source_list = $syncStorage->listAll();
    $storage_comparer = new StorageComparer($syncStorage, $this->configStorageActive);

    $changes = !(empty($source_list) || !$storage_comparer->createChangelist()->hasChanges());
    return $changes;
  }

  /**
   * Returns when the last notification was sent.
   *
   * @return mixed
   *   Timestamp or false.
   */
  public function getLastNotificationSent() {
    return $this->state->get('config_notify.last_sent');
  }

  /**
   * Sets the last notification time.
   *
   * @param int $time
   *   Time to set.
   */
  public function setLastNotification($time) {
    $this->state->set('config_notify.last_sent', $time);
  }

  /**
   * Sends slack notification.
   *
   * @param string $message
   *   Message to send to slack.
   *
   * @return bool
   *   Whether the message was sent or not.
   */
  public function notifySlack($message) {
    if ($this->moduleHandler->moduleExists('slack')) {
      $response = \Drupal::service('slack.slack_service')->sendMessage($message);
      return ($response && RedirectResponse::HTTP_OK == $response->getStatusCode());
    }

    return FALSE;
  }

  /**
   * Sends email notification.
   *
   * @param string $message
   *   Message to send via email.
   * @param string $email
   *   Recipient of email if different from site administrator.
   *
   * @return bool
   *   Whether the message was sent or not.
   */
  public function notifyEmail($message, $email = NULL) {
    $module = 'config_notify';
    $key = 'config_notify';
    $to = empty($email) ? $this->configFactory->get('system.site')->get('mail') : $email;
    $params['message'] = $message;
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    $send = TRUE;

    $result = $this->pluginManagerMail->mail($module, $key, $to, $langcode, $params, NULL, $send);
    return (bool) $result['result'];
  }

}
